import 'dart:convert';
import 'dart:io';
import 'dart:typed_data';

import 'package:flutter/foundation.dart';
import 'package:http/http.dart' as http;
import 'package:ocg_app/ocg_app.dart';

import 'package:firebase_storage/firebase_storage.dart';

abstract class UploadRepository {
  Future<AppResponse<String>> storeFile(File file,
      {bool compress = true, @required String path});
}

class UploadTestRepository extends UploadRepository {
  @override
  Future<AppResponse<String>> storeFile(File file,
      {bool compress = true, String path}) async {
    return AppResponse.named(data: "123");
  }
}

class UploadNetoworkRepository extends UploadRepository {
  final http.BaseClient client;

  UploadNetoworkRepository(this.client);
  @override
  Future<AppResponse<String>> storeFile(File file,
      {bool compress = true, String path}) async {
    debugPrint("Upload ");
    try {
      String url = "/files/";
      var bytes = await file.readAsBytes();
      String body = base64UrlEncode(bytes);
      debugPrint("Base 64 $body");
      var response = await client.post(url,
          body: jsonEncode({
            "file": body,
            "metadata": jsonEncode({}),
          }),
          headers: ApiConfig.headers);
      var json = jsonDecode(response.body);
      String fileUrl = "/files/${json["file"]}";
      return AppResponse.named(data: fileUrl);
    } catch (e) {
      debugPrint("Upload Error $e");
      return AppResponse.named(error: e.toString());
    }
  }
}

class UploadFirebaseRepository extends UploadRepository {
  @override
  Future<AppResponse<String>> storeFile(File file,
      {bool compress = true, String path}) async {
    try {
      Uint8List data = await file.readAsBytes();
      StorageReference reference = FirebaseStorage.instance.ref().child(path);
      StorageUploadTask uploadTask = reference.putData(
        data,
      );
      StorageTaskSnapshot snapshot = await uploadTask.onComplete;
      String imagePath = (await snapshot.ref.getDownloadURL()).toString();
      return AppResponse.named(data: imagePath);
    } catch (e) {
      debugPrint("Error $e");
      return AppResponse.named(error: "Some error occurred");
    }
  }
}
